%% Numerische Integration mit Matlab
%% am Beispiel eines Einmassenschwingers
%
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut fuer Technische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%

clear all
close all

% Notwendige Ordner zum Matlabpfad hinzufuegen
nameFile = mfilename('fullpath');
pathFile = fileparts(nameFile);
pathEfun = fullfile(pathFile,'eFunktion');
pathInt  = fullfile(pathFile,'einMassenSchwinger');
addpath(pathEfun,pathInt)

%% Gesucht: Zeitverlauf y(t)

% Simulationszeit
t0 = 0; % [s]
t1 = 70;% [s]

% Anfangsbedingungen y(t0) 
y0 = 1;  % [m]
dy0 = 0; % [m/s]

x0 = [y0; dy0];

% Schrittweite
h = 0.5; % 0.01 0.1 0.5


%% gedaempfter Einmassenschwinger
% m * d2/dt(y ) - d * d/dt(y) - k * y = 0

%% ungedaempfter Einmassenschwinger
% m * d2/dt(y ) - k y = 0; => d = 0

%% Zustandsraumdarstellung:
% d/dt(x) = f(x) = A*x mit x = [x1 x2]
% und x1 = y, x2 = d/dt(y)

%% exakte Loesung
% Zeit

texakt = simTime2vector(t0,t1,0.01);
Yexakt = emsExakt(texakt,x0);

hFig = figure(1);
plot(texakt,Yexakt,'DisplayName','exakte Loesung')

createFigureLabels(hFig,'Einmassenschwinger',...
                        'Zeit [s]',...
                        'Position [m]');



%% Integration mit Euler-Vorwaerts



[teEeuler,XeEuler] = eEuler(@emsSS,[t0 t1],x0,h);
Yint = emsExakt(teEeuler,x0); % Auswertung der Exakten Lösung zu gegebenen Zeitpunkten
% Vgl. Euler-Vorwaerts Zeitintegration mit exakter Loesung
hFig = figure(1);
plot(teEeuler,XeEuler(1,:),'DisplayName','Euler-Vorwaearts')
hold all
plot(texakt,Yexakt,'DisplayName','exakte Loesung')

createFigureLabels(hFig,'Einmassenschwinger',...
                        'Zeit [s]',...
                        'Position [m]')


% Absoluter Integrationsfehler Euler-Vorwaerts
hFig = figure(2);
plot(teEeuler,XeEuler(1,:)-Yint,'DisplayName','$e_{abs} = y_{euler} - y_{exakt}$')
createFigureLabels(hFig,'absoluter Integrationsfehler Euler-Vorwaerts',...
                        'Zeit [s]',...
                        'Fehler')

% Relativer Fehler Euler-Vorwaerts
hFig = figure(3);
plot(teEeuler,abs(XeEuler(1,:)+100-(Yint+100))./(Yint+100)*100,'DisplayName','$e_{rel} = \frac{\|y_{euler} - y_{exakt}\|}{\|y_{exakt}\|}$')

createFigureLabels(hFig,'relativer Integrationsfehler Euler-Vorwaerts',...
                        'Zeit [s]',...
                        'Fehler [\%]')


%% Integration mit Euler-Rueckwaerts

[tiEeuler,XiEuler] = iEuler(@emsSS,[t0 t1],x0,h);
Yint = emsExakt(tiEeuler,x0); % Auswertung der Exakten Lösung zu gegebenen Zeitpunkten

% Vgl. Euler-RueckwaertsZeitintegration mit exakter Loesung
hFig = figure(4);
plot(tiEeuler,XiEuler(1,:),'DisplayName','Euler-Rueckwaerts')
hold all
plot(texakt,Yexakt,'DisplayName','exakte Loesung')

createFigureLabels(hFig,'Einmassenschwinger',...
                        'Zeit [s]',...
                        'Position [m]')

% Absoluter Integrationsfehler Euler-Rueckwaerts
hFig = figure(5);
plot(tiEeuler,XiEuler(1,:)-Yint,'DisplayName','$e_{abs} = y_{euler} - y_{exakt}$')

createFigureLabels(hFig,'absoluter Integrationsfehler Euler-Rueckwaerts',...
                        'Zeit [s]',...
                        'Fehler')

% Relativer Fehler Euler-Rueckwaerts
hFig = figure(6);
plot(tiEeuler,abs(XiEuler(1,:)+100-(Yint+100))./(Yint+100)*100,'DisplayName','$e_{rel} = \frac{\|y_{euler} - y_{exakt}\|}{\|y_{exakt}\|}$')

createFigureLabels(hFig,'relativer Integrationsfehler Euler-Rueckwaerts',...
                        'Zeit [s]',...
                        'Fehler [\%]')

%% Integration mit ODE45 (Matlab built-in)
% variable Schrittweite

sol = ode45(@emsSS,[t0, t1],x0); % length(sol.x)

t = simTime2vector(t0,t1,h);
Xruku4 = deval(t,sol);
Yruku4 = Xruku4(1,:);

Yint = emsExakt(t,x0); % Auswertung der Exakten Lösung zu gegebenen Zeitpunkten


% Vgl. ode45 Zeitintegration mit exakter Loesung
hFig = figure(7)
plot(texakt,Yexakt,'DisplayName','exakte Loesung')
hold all
plot(t,Yruku4,'DisplayName','Runge-Kutta 4')

createFigureLabels(hFig,'Einmassenschwinger',...
                        'Zeit [s]',...
                        'Position [m]')

% Absoluter Integrationsfehler ode45
hFig = figure(8)
plot(t,Yruku4-Yint,'DisplayName','$e_{abs} = y_{ruku4}-y_{exakt}$')

createFigureLabels(hFig,'absoluter Integrationsfehler Runge-Kutta 4',...
                        'Zeit [s]',...
                        'Fehler')

% Relativer Fehler Runge-Kutta 4
hFig = figure(9)
plot(t,abs(Yruku4+100-(Yint+100))./(Yint+100)*100,'DisplayName','$e_{rel} = \frac{\|y_{ruku4} - y_{exakt}\|}{\|y_{exakt}\|}$')

createFigureLabels(hFig,'relativer Integrationsfehler Runge-Kutta 4',...
                        'Zeit [s]',...
                        'Fehler [\%]')

% Variable Schrittweitensteuerung
hFig = figure(10)
bar(diff(sol.x))
grid on
title('Automatische Schrittweitensteuerung ode45')
xlabel('Integrationsschritte')
ylabel('Schrittweite [s]')




