%% Loesung linearer Gleichungssysteme am Beispiel eines elastischen Balkens
%
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut fuer Technische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%
clear all
close all

% Notwendige Ordner zum Matlabpfad hinzufuegen
nameFile = mfilename('fullpath');
pathFile = fileparts(nameFile);
pathLGS = fullfile(pathFile,'lgsSolve');
addpath(pathLGS)

% Parameter
l = 1000; % Laenge [mm]
EI = 2e10; % Biegesteifigkeit [N*mm^2]
q0 = 2; % Streckenlast [N/mm]
% Mit den oben genannten Parametern betraegt die maximale Durchbiegung
% am Ende des Balkens: 12.5 mm

% Zahlenbeispiel aus 
% Technische Mechanik, J. Dankert, H. Dankert, Springer Vieweg, 7. Auflage
% Kapitel 18.1.2: Biegelinie bei konstanter Biegesteifigkeit, S.275-278

Diskretisierung = [4 20 50 100 500 1000 2000 5000 10000]; % nA Anzahl Balkenelemente
nDisk = length(Diskretisierung);

%% Numerische Berechnung der Knotenverschiebungen
% Aufbau LGS
[K, F] = Biegelinie(100,l,EI,q0);
spy(K) % Visualisierung der Matrix K

% Matlab built-in Loeser
Durchbiegung = NaN(1,nDisk);
Rechenzeit = NaN(1,nDisk);
for i = 1:1:nDisk
    nA = Diskretisierung(i);
    [K, F] = Biegelinie(nA,l,EI,q0);

    start = tic;
    vBuiltIn = K\F;
    Rechenzeit(i) = toc(start);
    Durchbiegung(i) = vBuiltIn(end-2);
end

% Eigene Gauss Implementierung
nGauss = 5; % Diskretisierung nur bis 500 Elemente => Rechenzeit!
Durchbiegung1 = NaN(1,nGauss);
Rechenzeit1 = NaN(1,nGauss);

for i = 1:1:nGauss
    nA = Diskretisierung(i);
    [K, F] = Biegelinie(nA,l,EI,q0);
    % Gauss
    [v, tBalken] = GaussMitPivot(K,F);
    Rechenzeit1(i) = tBalken;
    Durchbiegung1(i) = v(end-2);
end

%% Visualisierung der Ergebnisse
% Vergleich Matlab build-in Loeser mit exakter Loesung
figure(1)
plot(12.5*ones(1,nDisk),'k','DisplayName','exakte Loesung')
hold on
plot(Durchbiegung,'LineWidth',2,'Color','r','Marker','*','MarkerEdgeColor','b','DisplayName','num. Loesung')
title('Durchbiegung')
ylabel('Durchbiegung [mm]')
xlabel('Balkenelemente')
set(gca,'xtick',1:1:nDisk,'xticklabel',Diskretisierung)
legend show

% Relativer Fehler im Vergleich
vexakt = 12.5*ones(1,nGauss);
relFehlMldivide = abs(Durchbiegung(1:nGauss)-vexakt)./vexakt*100;
relFehlGauss = abs(Durchbiegung1-vexakt)./vexakt*100;
figure(2)
title('Relativer Fehler')
plot(relFehlMldivide,'LineWidth',2,'Color','r','DisplayName','mldivide')
hold on
plot(relFehlGauss,'LineWidth',2,'LineStyle','--','Color','b','DisplayName','Gauss')
title('Vergleich der Verfahren')
ylabel('relativer Fehler [%]')
xlabel('Balkenelemente')
set(gca,'xtick',1:1:nDisk,'xticklabel',Diskretisierung)
legend show

% Rechenzeit
figure(3)
title('Rechenzeit im Vergleich')
plot(Rechenzeit(1:nGauss),'LineWidth',2,'Color','r','DisplayName','Loesung mldivide')
hold on
plot(Rechenzeit1,'LineWidth',2,'Color','b','DisplayName','Loesung Gauss')
ylabel('Rechenzeit [s]')
xlabel('Balkenelemente')
set(gca,'xtick',1:1:nDisk,'xticklabel',Diskretisierung)
legend show


%% Kondition
nDisk = length(Diskretisierung);
Kondition = NaN(1,nDisk);
RechenzeitKond = NaN(1,nDisk);
for i = 1:1:nDisk
    nA = Diskretisierung(i);
    [K, F] = Biegelinie(nA,l,EI,q0);
    tstart = tic;
    Kondition(i) = cond(K,inf); % Siehe Hilfe zu cond() fuer Auswahl der Norm
    RechenzeitKond(i) = toc(tstart);
end

figure(4)
loglog(Diskretisierung,Kondition,'LineWidth',2,'Color','r','Marker','*','MarkerEdgeColor','b') % Vgl plot-Befehl
title('Kondition von K')
ylabel('cond(K)')
xlabel('Balkenelemente')
% set(gca,'xtick',1:1:nDisk,'xticklabel',Diskretisierung)
% Rechenzeit Konditionsberechnung
figure(5)
plot(RechenzeitKond,'LineWidth',2,'Color','r','Marker','*','MarkerEdgeColor','b')
title('Rechenzeit Konditionsberechnung')
ylabel('Rechenzeit [s]')
xlabel('Balkenelemente')
set(gca,'xtick',1:1:nDisk,'xticklabel',Diskretisierung)
